# モジュール読み込み ******************************************************
import time
import datetime
import cv2
import threading
# ｔime:時間に関するPython標準モジュールです。
# dateｔime:日付と時刻に関するPython標準モジュールです。
# cv2:OpenCVのPythonモジュールです。
# threading:マルチスレッド処理に関する標準モジュールです。

# 顔認識 ****************************************************************
face_cascade = \
        cv2.CascadeClassifier('/home/kuro/kurosSIRO/haarcascades/haarcascade_frontalface_alt.xml')
# 顔認識学習モデル（カスケード分類器）をファイル名で指定して読み込んでいます。

cap = cv2.VideoCapture(0)
# クラスのインスタンス生成

cap.set(3, 640)  # 画像サイズ WIDTH
cap.set(4, 480)  # 画像サイズ HEIGHT
# カメラの解像度を設定しています。
# 解像度を大きくすると画像が鮮明になりますが処理動作が遅くなります。

cap.set(cv2.CAP_PROP_FPS, 30)
# FPSを設定しています。
# 使用したUSBカメラはFPS30で固定なので設定してもあまり意味がないのですが一応明示しています。

cap.set(cv2.CAP_PROP_BUFFERSIZE, 1)
# バッファサイズを最小の１に設定しています。
# こうすることでバッファ分のフレームの遅延がなくなります。

minW = 50
minH = 50
# 認識可能な顔のサイズを設定しています。
# 大きくすると遠くの顔が認識できなくなりますが誤認識が少なくなります。
# 逆に小さくすると遠くの顔も認識できる様になりますが壁の模様を顔と間違える等の誤認識が増えます。

recognizer = cv2.face.LBPHFaceRecognizer_create()
# 顔判別クラスのインスタンス生成

recognizer.read('/home/kuro/kurosSIRO/trainer.yml')
# 作成した顔判別機械学習モデルを読み込んでいます。

font = cv2.FONT_HERSHEY_SIMPLEX
# 画像に文字を記載する際のフォントを指定しています。

face_id = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
# 顔判別カウンターです。
# 判別した顔のIDをカウントアップしていきます。
# IDNo.　0: 自分　1：妻　2：長男　3：次男　9:不審者　としています。　IDNo.4~8は未割り当てです。

thread_loop = 1
# サブスレッドの終了を制御する為のフラグです。

ret, frame = cap.read()


def siro_facerecog():
    # 顔認識処理を行う関数です。

    global thread_loop, face_id

    while (thread_loop):
        # メイン処理終了時にthread_loopを0にしてループから脱出させてサブスレッドを終了させます。

        ret, frame = cap.read()
        # カメラ画像を取得しています。

        grayimg = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        # 顔認識を行う前に画像をモノクロ処理して情報量を減らしています。

        facerect = face_cascade.detectMultiScale(grayimg, minSize=(minW, minH))
        # 顔認識を行っています。

        if len(facerect) > 0:
            # 顔を見つけたら

            for (x, y, w, h) in facerect:  # 顔を一つ一つ確認
                cv2.rectangle(frame, (x, y), (x+w, y+h), (0, 255, 0), 2)  # 顔検出箇所を四角で囲む

                id_count, confidence = recognizer.predict(grayimg[y:y+h, x:x+w])
                # 顔判別を行います。

                cv2.putText(frame, str(id_count), (x+5, y-5), font, 1, (255, 255, 255), 2)
                cv2.putText(frame, str(round(confidence, 1)), (x+5, y+h-5), font, 1, (255, 255, 0), 1)
                # 判別した顔のIDNo.と顔判別の信頼度を画面に表示します。

                if (confidence < 100):
                    # 顔判別の信頼度が一定以上であれば顔判別カウンターをカウントアップします。
                    face_id[id_count] = face_id[id_count]+2  # カウントアップ

                else:
                    # 顔判別の信頼度が低い場合は不審者と判断します。
                    face_id[9] = face_id[9]+2  # 不審者(IDNo.9)をカウントアップ

        for i in range(10):
            face_id[i] = face_id[i]-1
            # 顔がなくなるとカウントを１ずつ減らしていきます。

            if (face_id[i] > 10):
                face_id[i] = 10  # 上限１０
            if (face_id[i] < 0):
                face_id[i] = 0  # 下限０
            # カウントは0〜10(最大)としています。

        cv2.imshow('camera capture', frame)
        # カメラ画像をウインドウに表示させています。

        cv2.waitKey(100)
        # 処理周期を100msecに設定しています。

    cap.release()
    cv2.destroyAllWindows()
    print("face recognition stop.....",)
    # サブスレッド終了時の処理です。


th_facerecog = threading.Thread(target=siro_facerecog, daemon=True)
th_facerecog.start()  # サブスレッド開始
print("face recognition start.....",)
# 顔認識処理は常時処理且つ処理が重いので、メイン処理に影響を与えないようサブスレッドで処理します。


def siro_camera():
    # 写真撮影の関数を定義しています。

    now = datetime.datetime.now()
    # 日付と時刻を取得しています。

    filename = "/home/kuro/kurosSIRO/data/picture/" + now.strftime("%Y-%m-%d_%H-%M-%S")+".jpg"
    # 日付と時刻からファイル名を作成しています。

    ret, frame = cap.read()  # 画像取得
    cv2.imwrite(filename, frame)  # 画像保存
    # カメラ画像を取得しファイル名を指定して保存しています。


# メイン処理 *************************************************************
try:
    while True:

        print("写真を取ります。")
        siro_camera()
        # サブスレッドで顔認識処理を行いながら、メインスレッドで写真撮影を行います。

        time.sleep(5)
        # 5秒ごとに撮影します。

# 終了処理　**************************************************************
# Ctrl + C キーボード割り込み
except KeyboardInterrupt as reigai:
    print("Ctrl C ........")

finally:  # 終了処理

    print("SIRO stopping............")

    thread_loop = 0
    # サブスレッド終了フラグを下げてサブスレッド（顔認識処理）を終了させます。
    th_facerecog.join()
    # サブスレッドの完了を待っています。

# End of program ●▲■●▲■●▲■●▲■■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■
